#include "mbed.h"
#include "DHT.h"
#include "ESP-call.h"
#include "C12832.h"
#include "pitches.h"
#include "time_struct.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

Mutex mutex;


char rain[10];
char time1[11];
    
TIME current_time = {0,0,0};
TIME alarm_time = {0,0,0};


DHT dht_sensor(A0, SEN11301P); 
AnalogIn illu_sensor(A1);
AnalogIn flame_sensor(A2);
C12832 lcd(D11, D13, D12, D7, D10); // lcd = (MOSI, SCK, RESET, A0, nCS)
PwmOut buzzer(PTA1);        // buzzer = PTA1
DigitalOut red(D4);
DigitalOut yellow(D5);
DigitalOut blue(D6);



void wifiConnect(void)
{
    NodeMCU_PGM http_client[] = {
        "", 1,
        "wifi.setmode(wifi.STATION)", 1,
        "", 1,
        "station_cfg={}", 1,
        "station_cfg.ssid=\"in\"", 1,
        "station_cfg.pwd=\"87654321\"", 1,
        "station_cfg.save=false", 1,
        "wifi.sta.config(station_cfg)", 1,
        "", 1,  
        "wifi.sta.connect()", 80, // long delay here!
        "", 1,
        "print(wifi.sta.status())", 1,
        "print(wifi.sta.getip())", 1,
        NULL, 0, 
    };
    ESP_call_multi(http_client);
}
int t;
void getWeather(void)
{
    NodeMCU_PGM weather[] = {
        "", 1,
        "socket=net.createConnection(net.TCP, 0)", 1,
        "socket:on(\"receive\", function(sck, c) print(c) end)", 1,
        "socket:connect(80,\"api.openweathermap.org\")", 1,
        "socket:send(\"GET /data/2.5/weather?id=1835848&APPID=683a6b20feaa45aa163c4b290a4c4110 HTTP/1.1\\r\\nHost: api.openweathermap.org\\r\\nConnection: close\\r\\nAccept: */*\\r\\n\\r\\n\")", 50, 
        NULL, 0, 
    };
    ESP_call_multi(weather);
    char *plp[2] ={ESP_recv_buffer, 0};
    
    plp[1] = strstr(plp[0], "dt\":");
    for(int i =0; i<10; i++)
    {
        time1[i] = *(plp[1] + i+4);
    }
    time1[10] = '\0';
    set_time(atoi(time1));
    PC.printf("\r\ntime: %s\r\n", time1);
    plp[1] = strstr(plp[0], "main\":\"");
    for(int i =0; i<10; i++)
    {
        if(*(plp[1] + i+7) == '\"') 
        {
            rain[i] = '\0';
            break;
        }
        rain[i] = *(plp[1] + i+7);
    }
    ESP.printf("\r\nweather=\"%s\"\r\n", rain);
    
}

void thread_clock(void)
{
    while(true)
    {
        mutex.lock();
        time_t s = time(NULL);
        current_time.s = s%60;
        current_time.m = s/60%60;
        current_time.h = (s/60/60%24 + 9)%24;
        mutex.unlock();
        ThisThread::sleep_for(2000);
    }
}
NodeMCU_PGM http_server[] = {
    "", 1,
    "socket:on(\"receive\", function(sck, c) sck:close() end)", 1,
    "temp=0", 1,
    "humi=0", 1,
    "illu=0", 1,
    "flame=0", 1,
    "http_resp=\"HTTP/1.0 200 OK\\r\\nContent-Type: text/html\\r\\n\\r\\n\"", 1,
    "html_main1 = \"<h1>Embeded project</h1>\"", 1,
    "html_weather = \"<h1>weather:%s</h1>\"", 1,
    "html_temp = \"<h1>Temperature(C): %4.2f</h1>\"", 1,
    "html_humi = \"<h1>Humidity : %4.2f</h1>\"", 1,
    "html_illu = \"<h1>Illuminance : %.2f</h1>\"", 1,
    "html_flame = \"<h1>Flame : %.2f</h1>\"", 1,
    "html_main2 = \"<h1>ALARM</h1><form>\
    <input type=\\\"number\\\" name=\\\"hour\\\" min=\\\"0\\\" max=\\\"23\\\" step=\\\"1\\\">h\
    <input type=\\\"number\\\" name=\\\"min\\\" min=\\\"0\\\" max=\\\"59\\\" step=\\\"1\\\">m\
    <input type=\\\"submit\\\">\
    </form>\"", 1,
    "srv = net.createServer(net.TCP)", 1,
    "srv:listen(80, function(conn)", 1,
    "   conn:on(\"receive\", function(sck, payload)", 1,
    "       print(payload)", 1,
    "       print(\"~\")", 1,
    "       sck:send(http_resp)", 1,
    "       sck:send(html_main1)", 1,
    "       sck:send(string.format(html_weather, weather))", 1,
    "       sck:send(html_main2)", 1,
    "       sck:send(string.format(html_temp, temp))", 1,
    "       sck:send(string.format(html_humi, humi))", 1,
    "       sck:send(string.format(html_illu, illu))", 1,
    "       sck:send(string.format(html_flame, flame))", 1,
    "       end)", 1,
    "   conn:on(\"sent\", function(sck) sck:close() end)", 1,
    "end)", 1,
    NULL, 0,
};




typedef struct info{
    //DHT.h  DHT ReadTemperature(FARENHEIT), ReadHumidity()
    float temp; //온도
    float humi; //습도
    //AnalogIn read()
    float illu;  //조도
    float flame; //불꽃 감지
}INFO;

INFO info = {0,0,0,0};


int length = 5;

float frequency[] = {
    NOTE_E6, NOTE_E6, 0, NOTE_E6, 0, NOTE_C6, NOTE_E6, 0, NOTE_G6, 0, 0,
    0, NOTE_G5, 0, 0, 0, NOTE_C6, 0, 0, NOTE_G5, 0, 0, NOTE_E5, 0, 0,
    NOTE_A5, 0, NOTE_B5, 0, NOTE_AS5, NOTE_A5, 0, NOTE_G5, NOTE_E6, NOTE_G6,
    NOTE_A6, 0, NOTE_F6, NOTE_G6, 0, NOTE_E6, 0, NOTE_C6, NOTE_D6, NOTE_B5,
    0, 0, NOTE_C6, 0, 0, NOTE_G5, 0, 0, NOTE_E5, 0, 0, NOTE_A5, 0, NOTE_B5,
    0, NOTE_AS5, NOTE_A5, 0, NOTE_G5, NOTE_E6, NOTE_G6, NOTE_A6, 0, NOTE_F6,
    NOTE_G6, 0, NOTE_E6, 0, NOTE_C6, NOTE_D6, NOTE_B5, 0, 0
};

float beat[] = {
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 9, 9, 9, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12, 12, 12, 12, 12, 9, 9, 9, 12, 12, 12, 12, 12, 12, 12,
    12, 12, 12, 12, 12,
};
Thread th1, th2, th3, th4, th5, th6;
void thread_illu(void)
{
    while(true){
        int count = 0;
        if(info.temp > 25.0 || info.temp < 10.0) count++;
        if(info.humi < 50.0 || info.humi > 90.0) count++;
        if(info.illu > 0.8) count++;
        
        if(count >= 3){ blue = 0.0; yellow = 0.0; red = 1.0; }
        else if (count >= 2){ blue = 0.0; yellow = 1.0; red = 0.0; }
        else if (count >= 1){ blue = 1.0; yellow = 0.0; red = 0.0; }
        else{ blue = 0.0; yellow = 0.0; red = 0.0; } 
        ThisThread::sleep_for(2000);
    }
}   



void thread_get_and_print_info(void)
{

    int error;
    
    while(true)
    {
        mutex.lock();
        error = dht_sensor.readData();
        if(error == 0)
        {
            info.temp = dht_sensor.ReadTemperature(CELCIUS);
            info.humi = dht_sensor.ReadHumidity();
            info.illu = illu_sensor.read();
            info.flame = flame_sensor.read();
            ESP.printf("temp=%4.2f\r\n", info.temp);
            ESP.printf("humi=%4.2f\r\n", info.humi);
            ESP.printf("illu=%.2f\r\n", info.illu);
            ESP.printf("flame=%.2f\r\n", info.flame);
            mutex.unlock();
            ThisThread::sleep_for(2000);
        }
        else
        {
            info.illu = illu_sensor.read();
            info.flame = flame_sensor.read();
            PC.printf("ERROR\r\n");  
            ESP.printf("illu=%.2f\r\n", info.illu);
            ESP.printf("flame=%.2f\r\n", info.flame);
            mutex.unlock();
           ThisThread::sleep_for(2000);
        }
        
    }
}

void thread_flame_alarm(void)
{
    while(true)
    {
        mutex.lock();
        if(info.flame > 0.8)
        {
            for(int i = 0; i <= length; i++) {
                if(frequency[i] == 0) {
                    buzzer = 0.0;
                } else {
                    buzzer.period(1.0 / frequency[i]);  // period = (1.0 / frequency)
                    buzzer = 0.5;                       // duty cycle = 50%
                }
                thread_sleep_for(2500.0 / beat[i]);     // duration = (C / beat) ms
            }
        }
        else 
            buzzer = 0.0;
        mutex.unlock();
        
        ThisThread::sleep_for(2000);
    }
}
void thread_print_lcd(void)
{
    while(true)
    {
        mutex.lock();
        lcd.locate(0,10);
        lcd.printf("C:%d:%d:%d A:%d:%d:%d\r\n",
                    current_time.h,
                    current_time.m,
                    current_time.s,
                    alarm_time.h,
                    alarm_time.m,
                    alarm_time.s);
        mutex.unlock();
        ThisThread::sleep_for(1000);
    }
}
int length1 = 62;
void thread_water_alarm(void)
{
    while(true)
    {
        mutex.lock();
        if((alarm_time.h == current_time.h) && (alarm_time.m == current_time.m))
        {
            for(int i = 0; i <= length1; i++) {
                if(frequency[i] == 0) {
                    buzzer = 0.0;
                } else {
                    buzzer.period(1.0 / frequency[i]);  // period = (1.0 / frequency)
                    buzzer = 0.5;                       // duty cycle = 50%
                }
                thread_sleep_for(2500.0 / beat[i]);     // duration = (C / beat) ms
            }
        }
        else 
            buzzer = 0.0;
        mutex.unlock();
        
        ThisThread::sleep_for(2000);
    }
}

int main()
{
        // Config baudrate of PC and ESP
    PC.baud(115200);
    ESP.baud(115200);
    // Reset ESP
    PC.printf("\r\nReset ESP...\r\n");
    ESP_reset();
    // Setup ESP noecho mode
    PC.printf("Setup ESP noecho...\r\n");
    ESP_noecho();
    // Execute a NodeMCU program
    PC.printf("Execute a NodeMCU program...\r\n");
    wifiConnect();
    getWeather();
    ESP_call_multi(http_server);
    // Config ESP to PC receive mode
    PC.printf("\r\nESP receive mode...\r\n");
    //ESP.attach(&ISR_ESP_to_PC_new, Serial::RxIrq);

    lcd.cls();
    lcd.locate(0, 0);
    lcd.printf("Embeded project!");
    //thread_sleep_for(5000);
    th1.start(thread_get_and_print_info);
    th2.start(thread_flame_alarm);
    th3.start(thread_clock);
    th4.start(thread_print_lcd);
    th5.start(thread_illu);
    th6.start(thread_water_alarm);
    while(1) {
        char c;
        char *s1;
        ESP_recv_buffer_index = 0;
        for(;;) {
            if(ESP.readable()) {
                c = ESP.getc();
                if(c == '~') break;
                ESP_recv_buffer[ESP_recv_buffer_index] = c;
                ESP_recv_buffer_index++;
            }
        }
        ESP_recv_buffer[ESP_recv_buffer_index] = '\0';
        PC.printf("%s\r\n", ESP_recv_buffer);
        if(ESP_recv_buffer_index != 0) {
            if((s1 = strstr(ESP_recv_buffer, "hour")) != NULL ) {
                alarm_time.h = atoi(s1+5);
            } 
            if((s1 = strstr(ESP_recv_buffer, "min")) != NULL ) {
                alarm_time.m = atoi(s1+4);
            } 
        }
    }
}